@@ -17,11 +17,15 @@ ffi = build_ffi_for_binding(
         "common_key_derivation",
         "common_cryptor",
         "common_symmetric_key_wrap",
+        "seccertificate",
         "secimport",
         "secitem",
         "seckey",
         "seckeychain",
+        "secpolicy",
         "sectransform",
+        "sectrust",
+        "secure_transport",
     ],
     extra_link_args=[
         "-framework", "Security", "-framework", "CoreFoundation"
