@@ -40,7 +40,6 @@ static const int BIO_C_FILE_SEEK;
 static const int BIO_C_FILE_TELL;
 static const int BIO_TYPE_NONE;
 static const int BIO_TYPE_NBIO_TEST;
-static const int BIO_TYPE_BER;
 static const int BIO_TYPE_BIO;
 static const int BIO_TYPE_DESCRIPTOR;
 static const int BIO_FLAGS_READ;
@@ -82,12 +81,12 @@ int BIO_read(BIO *, void *, int);
 int BIO_gets(BIO *, char *, int);
 int BIO_write(BIO *, const void *, int);
 int BIO_puts(BIO *, const char *);
+int BIO_method_type(const BIO *);
 """
 
 MACROS = """
 /* These added const to BIO_METHOD in 1.1.0 */
 BIO *BIO_new(BIO_METHOD *);
-int BIO_set(BIO *, BIO_METHOD *);
 BIO_METHOD *BIO_s_mem(void);
 BIO_METHOD *BIO_s_file(void);
 BIO_METHOD *BIO_s_fd(void);
@@ -131,10 +130,6 @@ long BIO_set_write_buffer_size(BIO *, long);
 long BIO_set_buffer_size(BIO *, long);
 long BIO_set_buffer_read_data(BIO *, void *, long);
 long BIO_set_nbio(BIO *, long);
-
-/* The following was a macro in 0.9.8e. Once we drop support for RHEL/CentOS 5
-   we should move this back to FUNCTIONS. */
-int BIO_method_type(const BIO *);
 """
 
 CUSTOMIZATIONS = """
